@@ -46,6 +46,19 @@ static GThreadPool *thumbnail_pool;
 static GdkPixbuf *icons[NUM_ICONS];
 static GHashTable *cache_thumbnails; /* key=url, value=GdkPixbuf */
 
+static gboolean
+media_is_local (GrlMedia *media)
+{
+	const char *id;
+
+	id = grl_media_get_source (media);
+	if (g_strcmp0 (id, "grl-tracker-source") == 0 ||
+	    g_strcmp0 (id, "grl-filesystem") == 0 ||
+	    g_strcmp0 (id, "grl-bookmarks") == 0)
+		return TRUE;
+	return FALSE;
+}
+
 GdkPixbuf *
 totem_grilo_get_thumbnail_finish (GObject       *source_object,
 				  GAsyncResult  *res,
@@ -111,6 +124,32 @@ get_stream_thumbnail_cb (GObject *source_object,
 	g_object_unref (G_OBJECT (stream));
 }
 
+static void
+save_bookmark_thumbnail (GrlMedia *media,
+			 const char *uri)
+{
+	char *thumb_path, *thumb_url;
+	GrlRegistry *registry;
+	GrlSource *source;
+
+	if (g_strcmp0 (grl_media_get_source (media), "grl-bookmarks") != 0)
+		return;
+
+	thumb_path = gnome_desktop_thumbnail_path_for_uri (uri, GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
+	thumb_url = g_filename_to_uri (thumb_path, NULL, NULL);
+	g_free (thumb_path);
+	grl_media_set_thumbnail (media, thumb_url);
+	g_free (thumb_url);
+
+	registry = grl_registry_get_default ();
+	source = grl_registry_lookup_source (registry, "grl-bookmarks");
+	grl_source_store_sync (source,
+			       NULL,
+			       media,
+			       GRL_WRITE_NORMAL,
+			       NULL);
+}
+
 static void
 thumbnail_media_async_thread (GTask    *task,
 			      gpointer  user_data)
@@ -127,7 +166,16 @@ thumbnail_media_async_thread (GTask    *task,
 
 	media = GRL_MEDIA (g_task_get_source_object (task));
 	uri = grl_media_get_url (media);
+
 	mtime = grl_media_get_modification_date (media);
+	if (!mtime) {
+		GrlRegistry *registry;
+		GrlKeyID key_id;
+
+		registry = grl_registry_get_default ();
+		key_id = grl_registry_lookup_metadata_key (registry, "bookmark-date");
+		mtime = grl_data_get_boxed (GRL_DATA (media), key_id);
+	}
 
 	if (!uri || !mtime) {
 		g_task_return_new_error (task, G_IO_ERROR, G_IO_ERROR_FAILED, "URI or mtime missing");
@@ -144,6 +192,10 @@ thumbnail_media_async_thread (GTask    *task,
 	}
 
 	gnome_desktop_thumbnail_factory_save_thumbnail (factory, pixbuf, uri, g_date_time_to_unix (mtime));
+
+	/* Save the thumbnail URL for the bookmarks source */
+	save_bookmark_thumbnail (media, uri);
+
 	g_task_return_pointer (task, pixbuf, g_object_unref);
 	g_object_unref (task);
 }
@@ -188,18 +240,6 @@ thumbnail_media_cb (GObject      *source_object,
 	g_object_unref (task);
 }
 
-static gboolean
-media_is_local (GrlMedia *media)
-{
-	const char *id;
-
-	id = grl_media_get_source (media);
-	if (g_strcmp0 (id, "grl-tracker-source") == 0 ||
-	    g_strcmp0 (id, "grl-filesystem") == 0)
-		return TRUE;
-	return FALSE;
-}
-
 void
 totem_grilo_get_thumbnail (GObject             *object,
 			   GCancellable        *cancellable,
